'use strict';
const { CrudService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const _ = require('lodash');
const assert = require('assert');

// 初始化项目相关
class InstallService extends CrudService {
  constructor(ctx) {
    super(ctx, 'install');
    this.admin = this.ctx.model.System.Admin;
    this._tenant = this.ctx.model.System.Tenant;
    this.menu = this.ctx.model.System.Menu;
  }
  /**
   * 初始化
   */
  async index() {
    console.log('in function:install');
    // TODO,初始化管理员之类的
    // this.adminInit();
    // this.tenantInit();
    this.menuInit();
    this.tenantMenu();
  }

  /**
   * 管理员初始化,生成最高权限----root管理员,_tenant:分站标识,master是全站最高
   */
  async adminInit() {
    const has = await this.admin.count({ _tenant: 'master', is_root: true });
    if (has > 0) return;
    const data = {
      name: '总管理员',
      login_id: 'admin',
      password: {
        secret: '123456',
      },
      is_root: true,
    };
    await this.admin.create(data);
  }

  /**
   * 站点列表初始化
   */
  async tenantInit() {
    const has = await this._tenant.count({ _tenant: 'master' });
    if (has > 0) return;
    const data = {
      _tenant: 'master',
      name: '总管理站',
    };
    await this._tenant.create(data);
  }

  /**
   * 权限列表初始化
   */
  async menuInit() {
    const arr = [
      { title: '首页', disabled: false, sort: 999, route: '/', icon: 'el-icon-s-grid', can_delete: false },
      { title: '系统用户管理', disabled: false, sort: 990, route: '/users', icon: 'el-icon-user', can_delete: false },
      { title: '权限管理', disabled: false, sort: 980, route: '/menus', icon: 'el-icon-menu', can_delete: false },
      { title: '站点设置', disabled: false, sort: 970, route: '/setting', icon: 'el-icon-setting', can_delete: false },
    ];
    await this.menu.insertMany(arr);
  }

  /**
   * 初始化分站菜单
   */
  async tenantMenu() {
    const m_tenant = await this._tenant.findOne({ _tenant: 'master' });
    const menus = await this.menu.find({ can_delete: false });
    if (menus.length <= 0) return;
    const menus_ids = menus.map(i => JSON.parse(JSON.stringify(i._id)));
    m_tenant.menus = menus_ids;
    await m_tenant.save();
  }
}

module.exports = InstallService;
