'use strict';

const assert = require('assert');
const _ = require('lodash');
const moment = require('moment');
const { ObjectId } = require('mongoose').Types;
const { CrudService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;

class MenuService extends CrudService {
  constructor(ctx) {
    super(ctx, 'menu');
    this.model = this.ctx.model.System.Menu;
    this._tenant = this.ctx.model.System.Tenant;
    this.admin = this.ctx.model.System.Admin;
  }

  async delete({ id }) {
    let children = await this.dbFindChildren(id);
    children = children.map(i => ObjectId(i._id));
    children.push(ObjectId(id));
    const res = await this.model.deleteMany({ _id: children });
    return res;
  }

  async dbFindChildren(pid) {
    let toReturn = [];
    const res = await this.model.find({ pid }, '_id');
    toReturn = [ ...toReturn, ...res ];
    if (res.length > 0) {
      for (const i of res) {
        const { _id } = i;
        const list = await this.dbFindChildren(_id);
        toReturn = [ ...toReturn, ...list ];
      }
    }
    return toReturn;
  }

  async findProject() {
    const res = await this.model.find().sort({ sort: -1 });
    let list = [];
    if (res && res.length > 0) {
      list = await this.toFindChildren(res);
    }
    return list;
  }

  toFindChildren(list) {
    list = JSON.parse(JSON.stringify(list));
    const head = list.filter(f => !f.pid);
    if (head.length <= 0) throw new BusinessError(ErrorCode.DATA_INVALID, '需要将根目录加入整合函数的参数中');
    return this.findChildren(head, list);
  }

  findChildren(list, data) {
    for (const i of list) {
      let children = data.filter(f => f.pid === i._id);
      if (children.length > 0) children = this.findChildren(children, data);
      i.children = children;
    }
    return list;
  }

  async getUserMenu({ id }) {
    const user = await this.admin.findById(id);
    if (!user) throw new BusinessError(ErrorCode.USER_NOT_EXIST, '未找到用户信息!');
    const { _tenant } = user;
    if (!_tenant) throw new BusinessError(ErrorCode.DATA_INVALID, '未找到用户分站信息');
    const tenant = await this._tenant.findOne({ _tenant });
    if (!tenant) throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '未找到用户所在分站的信息');
    const { menus } = tenant;
    if (menus.length <= 0) throw new BusinessError(ErrorCode.DATA_INVALID, '未赋予用户任何权限');
    const mList = await this.model.find({ _id: menus.map(i => ObjectId(i)) });
    return this.toFindChildren(mList);

  }
}

module.exports = MenuService;
